/***
  BEGIN LICENSE
    
  Copyright (C) 2011 Mario Guerriero <mefrio.g@gmail.com>    
  This program is free software: you can redistribute it and/or modify it    
  under the terms of the GNU Lesser General Public License version 3, as published    
  by the Free Software Foundation.
    
  This program is distributed in the hope that it will be useful, but    
  WITHOUT ANY WARRANTY; without even the implied warranties of    
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR    
  PURPOSE.  See the GNU General Public License for more details.
    
  You should have received a copy of the GNU General Public License along    
  with this program.  If not, see <http://www.gnu.org/licenses/>    
  
  END LICENSE    
***/ 

namespace Appearance {
    
    class ThemeWidget : Gtk.Grid {
        
        Gtk.ComboBoxText window_combobox;
        Gtk.ComboBoxText widgets_combobox;
        Gtk.ComboBoxText cursor_combobox;
        Gtk.ComboBoxText icons_combobox;
        Gtk.ComboBoxText dock_theme_combobox;
        Gtk.ComboBoxText dock_hide_combobox;
        Gtk.Scale dock_size_scale;
        
        public ThemeWidget () {
            var system_label = new Gtk.Label ("<big><b>"+_("System:")+"</b></big>");
            system_label.set_alignment (0, (float)0.5);
            system_label.set_use_markup (true);
            
            var window_label = new Gtk.Label (_("Window:"));
            window_label.set_alignment (1, (float)0.5);
            window_combobox = new Gtk.ComboBoxText ();
            window_combobox.changed.connect (() => { on_window_changed (window_combobox.get_active_text ()); });
            
            var widgets_label = new Gtk.Label (_("Widgets:"));
            widgets_label.set_alignment (1, (float)0.5);
            widgets_combobox = new Gtk.ComboBoxText ();
            widgets_combobox.changed.connect (() => { on_widget_changed (widgets_combobox.get_active_id ()); });
            
            var cursor_label = new Gtk.Label (_("Cursor:"));
            cursor_label.set_alignment (1, (float)0.5);
            cursor_combobox = new Gtk.ComboBoxText ();
            cursor_combobox.changed.connect (() => { on_cursor_changed (cursor_combobox.get_active_id ()); });
            
            var icons_label = new Gtk.Label (_("Icons:"));
            icons_label.set_alignment (1, (float)0.5);
            icons_combobox = new Gtk.ComboBoxText ();
            icons_combobox.changed.connect (() => { on_icons_changed (icons_combobox.get_active_id ()); });
            
            var dock_label = new Gtk.Label ("<big><b>"+_("Dock:")+"</b></big>");
            dock_label.set_alignment (0, (float)0.5);
            dock_label.set_use_markup (true);
            
            var dock_theme_label = new Gtk.Label (_("Theme:"));
            dock_theme_label.set_alignment (1, (float)0.5);
            dock_theme_combobox = new Gtk.ComboBoxText ();
            dock_theme_combobox.changed.connect (() => { on_dock_theme_changed (dock_theme_combobox.get_active_id ()); });
            
            var dock_size_label = new Gtk.Label (_("Size:"));
            dock_size_label.set_alignment (1, (float)0.5);
            dock_size_scale = new Gtk.Scale.with_range (Gtk.Orientation.HORIZONTAL, 24, 128, 8);
            dock_size_scale.add_mark (48, Gtk.PositionType.BOTTOM, "48px");
            dock_size_scale.set_value_pos (Gtk.PositionType.LEFT);
            dock_size_scale.value_changed.connect (() => { on_dock_size_changed (dock_size_scale.get_value ()); });
            
            var dock_hide_label = new Gtk.Label (_("Hide mode:"));
            dock_hide_label.set_alignment (1, (float)0.5);
            dock_hide_combobox = new Gtk.ComboBoxText ();
            dock_hide_combobox.changed.connect (() => { on_dock_hide_changed (dock_hide_combobox.get_active_id ()); });
            
            var fake1_label = new Gtk.Label ("");
            var fake2_label = new Gtk.Label ("");
            
            attach (system_label, 1, 0, 1, 1);
            attach (window_label, 1, 1, 1, 1);
            attach (window_combobox, 2, 1, 1, 1);
            attach (widgets_label, 1, 2, 1, 1);
            attach (widgets_combobox, 2, 2, 1, 1);
            attach (cursor_label, 1, 3, 1, 1);
            attach (cursor_combobox, 2, 3, 1, 1);
            attach (icons_label, 1, 4, 1, 1);
            attach (icons_combobox, 2, 4, 1, 1);
            attach (dock_label, 1, 5, 1, 1);
            attach (dock_theme_label, 1, 6, 1, 1);
            attach (dock_theme_combobox, 2, 6, 1, 1);
            attach (dock_hide_label, 1, 7, 1, 1);
            attach (dock_hide_combobox, 2, 7, 1, 1);
            attach (dock_size_label, 1, 8, 1, 1);
            attach (dock_size_scale, 2, 8, 1, 1);
            attach (fake1_label, 0, 0, 1, 1);
            attach (fake2_label, 3, 0, 1, 1);
            
            system_label.set_hexpand(true);
            set_column_spacing (12);
            set_row_spacing (6);
            set_margin_top (12);
            set_column_homogeneous (true);
            
            load_comboboxes ();
        }
        
        public async void load_comboboxes () {
                    /* Load all window themes */
            var list = find_files ("/usr/share/themes/", null, "metacity-theme-1.xml");
            var current_window_theme = get_window_theme ();
            foreach (var file in list) {
                string file_content = load_file (file);
                string theme_name = file_content.split("<name>")[1].split("</name>")[0];
                window_combobox.append (file, theme_name);
                if (current_window_theme == theme_name)
                    window_combobox.set_active_id (file);
            }
            
            /* Load all widget themes */
            list = find_files ("/usr/share/themes/", null, ".theme");
            var current_widget_theme = get_widget_theme ();
            foreach (var file in list) {
                
                var keyfile = new KeyFile ();
                string file_content = load_file (file);
                size_t len = file_content.length;
                try {
                    keyfile.load_from_data (file_content, len, 0);
                } catch (GLib.KeyFileError err) {
                    warning ("%s", err.message);
                }
                try {
                    string name = keyfile.get_locale_string ("Desktop Entry", "Name");
                    if (name != null) {
                        widgets_combobox.append (file, name);
                        if (current_widget_theme == name)
                            widgets_combobox.set_active_id (file);
                    }
                } catch (GLib.KeyFileError err) {
                    warning ("%s", err.message);
                }
            }
            
            /* Warning : It is important that iconsare loaded AFTER cursor, because cursors inherit from icon files */
            
            /* Load all cursor themes */
            list = find_files ("/usr/share/icons/", null, "cursor.theme");
            var list2 = new List<string> ();
            var current_cursor_theme = get_cursor_theme ();
            foreach (var file in list) {
                
                var keyfile = new KeyFile ();
                string file_content = load_file (file);
                size_t len = file_content.length;
                try {
                    keyfile.load_from_data (file_content, len, 0);
                    string oldname = keyfile.get_string ("Icon Theme", "Inherits");
                    file_content = load_file ("/usr/share/icons/"+oldname+"/index.theme");
                    len = file_content.length;
                    keyfile.load_from_data (file_content, len, 0);
                    list2.append ("/usr/share/icons/"+oldname+"/index.theme");
                    string name = keyfile.get_locale_string ("Icon Theme", "Name");
                    if (name != null) {
                        cursor_combobox.append (file, name);
                        if (current_cursor_theme == oldname)
                            cursor_combobox.set_active_id (file);
                    }
                } catch (GLib.KeyFileError err) {
                    warning ("%s", err.message);
                }
            }
            
            /* Load all icons themes */
            list = find_files ("/usr/share/icons/", null, "index.theme");
            var current_icons_theme = get_icons_theme ();
            foreach (var file in list) {
                var keyfile = new KeyFile ();
                string file_content = load_file (file);
                size_t len = file_content.length;
                
                try {
                    keyfile.load_from_data (file_content, len, 0);
                } catch (GLib.KeyFileError err) {
                    warning ("%s", err.message);
                }
                try {
                    string name = keyfile.get_locale_string ("Icon Theme", "Name");
                    if (name != null) {
                        bool to_show = true;
                        foreach (var file2 in list2) {
                            if (file2 == file)
                                to_show = false;
                        }
                        if (to_show)
                            icons_combobox.append (file, name);
                        if (current_icons_theme == name)
                            icons_combobox.set_active_id (file);
                    }
                } catch (GLib.KeyFileError err) {
                    warning ("%s", err.message);
                }
            }
            
            /* Load Plank themes */
            list = find_plank_themes ("/usr/share/plank/themes/");
            var current_dock_theme = get_dock_theme ();
            foreach (var file in list) {
                dock_theme_combobox.append (file, file);
                if (current_dock_theme == "/usr/share/plank/themes/"+file)
                    dock_theme_combobox.set_active_id (file);
            }
            
            dock_hide_combobox.append ("0", _("Never"));
            dock_hide_combobox.append ("1", _("Intelligent"));
            dock_hide_combobox.append ("2", _("Always"));
            dock_hide_combobox.set_active_id ((string)get_dock_hide ());
            
            dock_size_scale.set_value (get_dock_size ());
        }
        
        public void on_window_changed (string theme_name) {
            var settings = new GLib.Settings ("org.gnome.desktop.wm.preferences");
            settings.set_string ("theme", theme_name);
            settings = new GLib.Settings ("org.pantheon.desktop.gala.appearance");
            settings.set_string ("theme", theme_name);
        }
        
        public string get_window_theme () {
            string theme_name = "";
            var settings = new GLib.Settings ("org.pantheon.desktop.gala.appearance");
            theme_name = settings.get_string ("theme");
            if (theme_name == "" || theme_name == null) {
                settings = new GLib.Settings ("org.gnome.desktop.wm.preferences");
                return settings.get_string ("theme");
            }
            else
                return theme_name;
        }
        
        public void on_widget_changed (string theme_path) {
        
            var keyfile = new KeyFile ();
            string file_content = load_file (theme_path);
            size_t len = file_content.length;
            try {
                keyfile.load_from_data (file_content, len, 0);
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
            var settings = new GLib.Settings ("org.gnome.desktop.interface");
            try {
                settings.set_string ("gtk-theme", keyfile.get_locale_string ("Desktop Entry", "Name"));
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
            try {
                string layout = keyfile.get_locale_string ("X-GNOME-Metatheme", "ButtonLayout");
                if (layout != null) {
                    settings = new GLib.Settings ("org.gnome.desktop.wm.preferences");
                    settings.set_string ("button-layout", layout);
                }
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
        }
        
        public string get_widget_theme () {
            var settings = new GLib.Settings ("org.gnome.desktop.interface");
            return settings.get_string ("gtk-theme");
        }
        
        public void on_icons_changed (string theme_path) {
            var keyfile = new KeyFile ();
            string file_content = load_file (theme_path);
            size_t len = file_content.length;
            try {
                keyfile.load_from_data (file_content, len, 0);
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
            var settings = new GLib.Settings ("org.gnome.desktop.interface");
            try {
                settings.set_string ("icon-theme", keyfile.get_locale_string ("Icon Theme", "Name"));
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
        }
        
        public string get_icons_theme () {
            var settings = new GLib.Settings ("org.gnome.desktop.interface");
            return settings.get_string ("icon-theme");
        }
        
        public void on_cursor_changed (string theme_path) {
            var keyfile = new KeyFile ();
            string file_content = load_file (theme_path);
            size_t len = file_content.length;
            try {
                keyfile.load_from_data (file_content, len, 0);
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
            var settings = new GLib.Settings ("org.gnome.desktop.interface");
            try {
                settings.set_string ("cursor-theme", keyfile.get_locale_string ("Icon Theme", "Inherits"));
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
        }
        
        public string get_cursor_theme () {
            var settings = new GLib.Settings ("org.gnome.desktop.interface");
            return settings.get_string ("cursor-theme");
        }
        
        
        public void on_dock_size_changed (double size) {
            var keyfile = new KeyFile ();
            string file_content = load_file (Environment.get_user_config_dir () + "/plank/dock1/settings");
            size_t len = file_content.length;
            try {
                keyfile.load_from_data (file_content, len, 0);
                keyfile.set_double ("PlankDockPreferences", "IconSize", size);
                var file = File.new_for_path (Environment.get_user_config_dir () + "/plank/dock1/settings");
                file.replace_contents (keyfile.to_data ().data, null, false, GLib.FileCreateFlags.REPLACE_DESTINATION, null);
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
        }
        public double get_dock_size () {
            var keyfile = new KeyFile ();
            string file_content = load_file (Environment.get_user_config_dir () + "/plank/dock1/settings");
            size_t len = file_content.length;
            try {
                keyfile.load_from_data (file_content, len, 0);
                return keyfile.get_double ("PlankDockPreferences", "IconSize");
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
            return 0;
        }
        
        public void on_dock_hide_changed (string hide_mode) {
            var keyfile = new KeyFile ();
            string file_content = load_file (Environment.get_user_config_dir () + "/plank/dock1/settings");
            size_t len = file_content.length;
            try {
                keyfile.load_from_data (file_content, len, 0);
                keyfile.set_string ("PlankDockPreferences", "HideMode", hide_mode);
                var file = File.new_for_path (Environment.get_user_config_dir () + "/plank/dock1/settings");
                file.replace_contents (keyfile.to_data ().data, null, false, GLib.FileCreateFlags.REPLACE_DESTINATION, null);
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
        }
        public string get_dock_hide () {
            var keyfile = new KeyFile ();
            string file_content = load_file (Environment.get_user_config_dir () + "/plank/dock1/settings");
            size_t len = file_content.length;
            try {
                keyfile.load_from_data (file_content, len, 0);
                return keyfile.get_string ("PlankDockPreferences", "HideMode");
            } catch (GLib.KeyFileError err) {
                warning ("%s", err.message);
            }
            return "0";
        }
        
        public void on_dock_theme_changed (string theme_name) {
            GLib.FileUtils.remove (Environment.get_user_config_dir () + "/plank/theme");
            GLib.FileUtils.symlink ("/usr/share/plank/themes/"+theme_name,Environment.get_user_config_dir () + "/plank/theme");
            
        }
        
        public string get_dock_theme () {
            string dock_theme = "";
            try {
                dock_theme = GLib.FileUtils.read_link(Environment.get_user_config_dir () + "/plank/theme");
            } catch (GLib.FileError err) {
                warning ("%s", err.message);
            }
            return dock_theme;
        }
        
        
        
        public string load_file (string file_path)
        {
            var file = GLib.File.new_for_path (file_path);
            try {
                uint8[] contents;
                bool success = file.load_contents (null, out contents, null);
                var contents_str = (string) contents;
                
                if (success && contents_str.length>0)
                {
                    return contents_str;
                }
            } catch (Error err) {
                warning ("%s", err.message);
            }
            return "error";
        }
        
        List<string> find_plank_themes (string path) {
            var directory = File.new_for_path (path);
            var keyfiles = new List<string> ();
            try {
                var enumerator = directory.enumerate_children (FileAttribute.STANDARD_NAME + "," + FileAttribute.STANDARD_TYPE, 0);
                FileInfo file_info;
                while ((file_info = enumerator.next_file ()) != null) {
                    if(file_info.get_file_type() == GLib.FileType.DIRECTORY) {
                        keyfiles.append(file_info.get_name());
                    }
                }
            }
            catch {
                warning(_(@"Unable to iterate over enumerated plug directory \"$path\"'s contents"));
            }
            return keyfiles;
        }
        
        // Find all files that have a specified extension
        List<string> find_files (string path, List<string>? keyfiles_list = null, string extension = "")
        {
            List<string>? keyfiles;
            if(keyfiles_list == null) {
                keyfiles = new List<string> ();
            }
            else {
                keyfiles = new List<string> ();
                foreach(var keyfile in keyfiles_list) {
                    keyfiles.append(keyfile);
                }
            }
            var directory = File.new_for_path (path);
            try {
                var enumerator = directory.enumerate_children (FileAttribute.STANDARD_NAME + "," + FileAttribute.STANDARD_TYPE, 0);
                FileInfo file_info;
                while ((file_info = enumerator.next_file ()) != null) {
                    string file_path = Path.build_filename(path, file_info.get_name());
                    if ((file_info.get_file_type() == GLib.FileType.REGULAR) && file_info.get_name().has_suffix (extension)) {
                        keyfiles.append(file_path);
                    }
                    else if(file_info.get_file_type() == GLib.FileType.DIRECTORY) {
                        keyfiles = find_files(file_path, keyfiles, extension);
                    }
                }
            }
            catch {
                warning(_(@"Unable to iterate over enumerated plug directory \"$path\"'s contents"));
            }
            return keyfiles;
        }
        
    }
}
